"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class VariantOperation(_ParentRestClient):
    """
    This class contains methods for the 'Operations - Variant Storage' webservices
    PATH: /{apiVersion}/operation
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(VariantOperation, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def configure_cellbase(self, data=None, **options):
        """
        Update Cellbase configuration.
        PATH: /{apiVersion}/operation/cellbase/configure

        :param str project: Project [organization@]project where project can
            be either the ID or the alias.
        :param bool annotation_update: Create and load variant annotations
            into the database.
        :param str annotation_save_id: Save a copy of the current variant
            annotation at the database.
        :param dict data: New cellbase configuration.
        """

        return self._post(category='operation', resource='configure', subcategory='cellbase', data=data, **options)

    def aggregate_variant(self, data=None, **options):
        """
        Find variants where not all the samples are present, and fill the
            empty values, excluding HOM-REF (0/0) values.
        PATH: /{apiVersion}/operation/variant/aggregate

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant aggregate params.
        """

        return self._post(category='operation', resource='aggregate', subcategory='variant', data=data, **options)

    def delete_variant_annotation(self, **options):
        """
        Deletes a saved copy of variant annotation.
        PATH: /{apiVersion}/operation/variant/annotation/delete

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str project: Project [organization@]project where project can
            be either the ID or the alias.
        :param str annotation_id: Annotation identifier.
        """

        return self._delete(category='operation', resource='delete', subcategory='variant/annotation', **options)

    def index_variant_annotation(self, data=None, **options):
        """
        Create and load variant annotations into the database.
        PATH: /{apiVersion}/operation/variant/annotation/index

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str project: Project [organization@]project where project can
            be either the ID or the alias.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant annotation index params.
        """

        return self._post(category='operation', resource='index', subcategory='variant/annotation', data=data, **options)

    def save_variant_annotation(self, data=None, **options):
        """
        Save a copy of the current variant annotation at the database.
        PATH: /{apiVersion}/operation/variant/annotation/save

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str project: Project [organization@]project where project can
            be either the ID or the alias.
        :param dict data: Variant annotation save params.
        """

        return self._post(category='operation', resource='save', subcategory='variant/annotation', data=data, **options)

    def configure_variant(self, data=None, **options):
        """
        Update Variant Storage Engine configuration. Can be updated at Project
            or Study level.
        PATH: /{apiVersion}/operation/variant/configure

        :param str project: Project [organization@]project where project can
            be either the ID or the alias.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Configuration params to update.
        """

        return self._post(category='operation', resource='configure', subcategory='variant', data=data, **options)

    def delete_variant(self, data=None, **options):
        """
        Remove variant files from the variant storage.
        PATH: /{apiVersion}/operation/variant/delete

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant delete file params.
        """

        return self._post(category='operation', resource='delete', subcategory='variant', data=data, **options)

    def aggregate_variant_family(self, data=None, **options):
        """
        Find variants where not all the samples are present, and fill the
            empty values.
        PATH: /{apiVersion}/operation/variant/family/aggregate

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant aggregate family params.
        """

        return self._post(category='operation', resource='aggregate', subcategory='variant/family', data=data, **options)

    def index_variant_family(self, data=None, **options):
        """
        DEPRECATED: integrated in index (DEPRECATED Build the family index).
        PATH: /{apiVersion}/operation/variant/family/index

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant family index params.
        """

        return self._post(category='operation', resource='index', subcategory='variant/family', data=data, **options)

    def index_variant(self, data=None, **options):
        """
        Index variant files into the variant storage.
        PATH: /{apiVersion}/operation/variant/index

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant index params.
        """

        return self._post(category='operation', resource='index', subcategory='variant', data=data, **options)

    def launcher_variant_index(self, data=None, **options):
        """
        Detect non-indexed VCF files in the study, and submit a job for
            indexing them.
        PATH: /{apiVersion}/operation/variant/index/launcher

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: .
        """

        return self._post(category='operation', resource='launcher', subcategory='variant/index', data=data, **options)

    def run_variant_julie(self, data=None, **options):
        """
        Transform VariantStats into PopulationFrequency values and updates the
            VariantAnnotation.
        PATH: /{apiVersion}/operation/variant/julie/run

        :param dict data: Julie tool params. Specify list of cohorts from
            multiple studies with {study}:{cohort}. (REQUIRED)
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str project: project.
        """

        return self._post(category='operation', resource='run', subcategory='variant/julie', data=data, **options)

    def repair_variant_metadata(self, data=None, **options):
        """
        Execute some repairs on Variant Storage Metadata. Advanced users only.
        PATH: /{apiVersion}/operation/variant/metadata/repair

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param dict data: Variant storage metadata repair params.
        """

        return self._post(category='operation', resource='repair', subcategory='variant/metadata', data=data, **options)

    def synchronize_variant_metadata(self, data=None, **options):
        """
        Synchronize catalog with variant storage metadata.
        PATH: /{apiVersion}/operation/variant/metadata/synchronize

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant storage metadata synchronize params.
        """

        return self._post(category='operation', resource='synchronize', subcategory='variant/metadata', data=data, **options)

    def prune_variant(self, data=None, **options):
        """
        Prune orphan variants from studies in a project.
        PATH: /{apiVersion}/operation/variant/prune

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param dict data: Variant prune params. Use dry-run to just generate a
            report with the orphan variants.
        """

        return self._post(category='operation', resource='prune', subcategory='variant', data=data, **options)

    def delete_variant_sample(self, data=None, **options):
        """
        Remove variant samples from the variant storage.
        PATH: /{apiVersion}/operation/variant/sample/delete

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant delete sample params.
        """

        return self._post(category='operation', resource='delete', subcategory='variant/sample', data=data, **options)

    def index_variant_sample(self, data=None, **options):
        """
        DEPRECATED You should use the new sample index method instead.
        PATH: /{apiVersion}/operation/variant/sample/index

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant sample index params.
        """

        return self._post(category='operation', resource='index', subcategory='variant/sample', data=data, **options)

    def variant_sample_index_configure(self, data=None, **options):
        """
        DEPRECATED You should use the new sample index configure method.
        PATH: /{apiVersion}/operation/variant/sample/index/configure

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool skip_rebuild: Skip sample index re-build.
        :param dict data: New SampleIndexConfiguration.
        """

        return self._post(category='operation', resource='configure', subcategory='variant/sample/index', data=data, **options)

    def delete_variant_score(self, **options):
        """
        Remove a variant score in the database.
        PATH: /{apiVersion}/operation/variant/score/delete

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str name: Unique name of the score within the study.
        :param bool resume: Resume a previously failed remove.
        :param bool force: Force remove of partially indexed scores.
        """

        return self._delete(category='operation', resource='delete', subcategory='variant/score', **options)

    def index_variant_score(self, data=None, **options):
        """
        Index a variant score in the database.
        PATH: /{apiVersion}/operation/variant/score/index

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant score index params. scoreName: Unique name
            of the score within the study. cohort1: Cohort used to compute the
            score. Use the cohort 'ALL' if all samples from the study where
            used to compute the score. cohort2: Second cohort used to compute
            the score, typically to compare against the first cohort. If only
            one cohort was used to compute the score, leave empty.
            inputColumns: Indicate which columns to load from the input file.
            Provide the column position (starting in 0) for the column with the
            score with 'SCORE=n'. Optionally, the PValue column with
            'PVALUE=n'. The, to indicate the variant associated with the score,
            provide either the columns ['CHROM', 'POS', 'REF', 'ALT'], or the
            column 'VAR' containing a variant representation with format
            'chr:start:ref:alt'. e.g.
            'CHROM=0,POS=1,REF=3,ALT=4,SCORE=5,PVALUE=6' or
            'VAR=0,SCORE=1,PVALUE=2'. resume: Resume a previously failed
            indexation.
        """

        return self._post(category='operation', resource='index', subcategory='variant/score', data=data, **options)

    def variant_secondary_annotation_index(self, data=None, **options):
        """
        Creates a secondary index using a search engine. If samples are
            provided, sample data will be added to the secondary index. (New!).
        PATH: /{apiVersion}/operation/variant/secondary/annotation/index

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str project: Project [organization@]project where project can
            be either the ID or the alias.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant secondary annotation index params.
        """

        return self._post(category='operation', resource='index', subcategory='variant/secondary/annotation', data=data, **options)

    def variant_secondary_sample_index(self, data=None, **options):
        """
        Build and annotate the sample index. (New!) .
        PATH: /{apiVersion}/operation/variant/secondary/sample/index

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant sample index params.
        """

        return self._post(category='operation', resource='index', subcategory='variant/secondary/sample', data=data, **options)

    def configure_variant_secondary_sample_index(self, data=None, **options):
        """
        Update SampleIndex configuration (New!).
        PATH: /{apiVersion}/operation/variant/secondary/sample/index/configure

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool skip_rebuild: Skip sample index re-build.
        :param dict data: New SampleIndexConfiguration.
        """

        return self._post(category='operation', resource='configure', subcategory='variant/secondary/sample/index', data=data, **options)

    def secondary_index_variant(self, data=None, **options):
        """
        DEPRECATED you should use the new annotation index method instead.
        PATH: /{apiVersion}/operation/variant/secondaryIndex

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str project: Project [organization@]project where project can
            be either the ID or the alias.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant secondary annotation index params.
        """

        return self._post(category='operation', resource='secondaryIndex', subcategory='variant', data=data, **options)

    def delete_variant_secondary_index(self, **options):
        """
        Remove a secondary index from the search engine for a specific set of
            samples.
        PATH: /{apiVersion}/operation/variant/secondaryIndex/delete

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str samples: Samples to remove. Needs to provide all the
            samples in the secondary index.
        """

        return self._delete(category='operation', resource='delete', subcategory='variant/secondaryIndex', **options)

    def setup_variant(self, data=None, **options):
        """
        Execute Variant Setup to allow using the variant engine. This setup is
            necessary before starting any variant operation.
        PATH: /{apiVersion}/operation/variant/setup

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant setup params.
        """

        return self._post(category='operation', resource='setup', subcategory='variant', data=data, **options)

    def delete_variant_stats(self, data=None, **options):
        """
        Deletes the VariantStats of a cohort/s from the database.
        PATH: /{apiVersion}/operation/variant/stats/delete

        :param dict data: Variant stats delete params. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='operation', resource='delete', subcategory='variant/stats', data=data, **options)

    def index_variant_stats(self, data=None, **options):
        """
        Compute variant stats for any cohort and any set of variants and index
            the result in the variant storage database.
        PATH: /{apiVersion}/operation/variant/stats/index

        :param dict data: Variant stats params. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='operation', resource='index', subcategory='variant/stats', data=data, **options)

    def delete_variant_study(self, data=None, **options):
        """
        Remove whole study from the variant storage.
        PATH: /{apiVersion}/operation/variant/study/delete

        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param dict data: Variant delete study params.
        """

        return self._post(category='operation', resource='delete', subcategory='variant/study', data=data, **options)




"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class Project(_ParentRestClient):
    """
    This class contains methods for the 'Projects' webservices
    PATH: /{apiVersion}/projects
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(Project, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def create(self, data=None, **options):
        """
        Create a new project.
        PATH: /{apiVersion}/projects/create

        :param dict data: JSON containing the mandatory parameters. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        """

        return self._post(category='projects', resource='create', data=data, **options)

    def search(self, **options):
        """
        Search projects.
        PATH: /{apiVersion}/projects/search

        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param str organization: Project organization.
        :param str id: Project [organization@]project where project can be
            either the ID or the alias.
        :param str name: Project name.
        :param str fqn: Project fqn.
        :param str description: Project description.
        :param str study: Study id.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param str internal_status: Filter by internal status.
        :param str attributes: Attributes.
        """

        return self._get(category='projects', resource='search', **options)

    def info(self, projects, **options):
        """
        Fetch project information.
        PATH: /{apiVersion}/projects/{projects}/info

        :param str projects: Comma separated list of projects
            [organization@]project up to a maximum of 100. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        """

        return self._get(category='projects', resource='info', query_id=projects, **options)

    def inc_release(self, project, **options):
        """
        Increment current release number in the project.
        PATH: /{apiVersion}/projects/{project}/incRelease

        :param str project: Project [organization@]project where project can
            be either the ID or the alias. (REQUIRED)
        """

        return self._post(category='projects', resource='incRelease', query_id=project, **options)

    def studies(self, project, **options):
        """
        Fetch all the studies contained in the project.
        PATH: /{apiVersion}/projects/{project}/studies

        :param str project: Project [organization@]project where project can
            be either the ID or the alias. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        """

        return self._get(category='projects', resource='studies', query_id=project, **options)

    def update(self, project, data=None, **options):
        """
        Update some project attributes.
        PATH: /{apiVersion}/projects/{project}/update

        :param dict data: JSON containing the params to be updated. It will be
            only possible to update organism fields not previously defined.
            (REQUIRED)
        :param str project: Project [organization@]project where project can
            be either the ID or the alias. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param bool include_result: Flag indicating to include the created or
            updated document result in the response.
        """

        return self._post(category='projects', resource='update', query_id=project, data=data, **options)




class OpencgaAuthorisationError(Exception):
    def __init__(self, message):
        super(OpencgaAuthorisationError, self).__init__(message)


class OpencgaInvalidToken(Exception):
    def __init__(self, message):
        super(OpencgaInvalidToken, self).__init__(message)



"""
WARNING: AUTOGENERATED CODE

    This code was generated by a tool.
    
    Manual changes to this file may cause unexpected behavior in your application.
    Manual changes to this file will be overwritten if the code is regenerated.
"""

from pyopencga_enterprise_enterprise.rest_clients._parent_rest_clients import _ParentRestClient


class File(_ParentRestClient):
    """
    This class contains methods for the 'Files' webservices
    PATH: /{apiVersion}/files
    """

    def __init__(self, configuration, token=None, login_handler=None, *args, **kwargs):
        super(File, self).__init__(configuration, token, login_handler, *args, **kwargs)

    def update_acl(self, members, action, data=None, **options):
        """
        Update the set of permissions granted for the member.
        PATH: /{apiVersion}/files/acl/{members}/update

        :param dict data: JSON containing the parameters to add ACLs.
            (REQUIRED)
        :param str action: Action to be performed [ADD, SET, REMOVE or RESET].
            Allowed values: ['SET ADD REMOVE RESET'] (REQUIRED)
        :param str members: Comma separated list of user or group ids.
            (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        options['action'] = action
        return self._post(category='files', resource='update', subcategory='acl', second_query_id=members, data=data, **options)

    def aggregation_stats(self, **options):
        """
        Fetch catalog file stats.
        PATH: /{apiVersion}/files/aggregationStats

        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str id: Comma separated list of file IDs up to a maximum of
            100. Also admits basic regular expressions using the operator '~',
            i.e. '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i'
            for case insensitive search.
        :param str uuid: Comma separated list file UUIDs up to a maximum of
            100.
        :param str name: Comma separated list of file names. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str path: Comma separated list of paths. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str uri: Comma separated list of uris. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str type: File type, either FILE or DIRECTORY.
        :param str bioformat: Comma separated Bioformat values. For existing
            Bioformats see files/bioformats.
        :param str format: Comma separated Format values. For existing Formats
            see files/formats.
        :param bool external: Boolean field indicating whether to filter by
            external or non external files.
        :param str status: Filter by status.
        :param str internal_status: Filter by internal status.
        :param str internal_variant_index_status: Filter by internal variant
            index status.
        :param str software_name: Software name.
        :param str directory: Directory under which we want to look for files
            or folders.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param str description: Description.
        :param str tags: Tags. Also admits basic regular expressions using the
            operator '~', i.e. '~{perl-regex}' e.g. '~value' for case
            sensitive, '~/value/i' for case insensitive search.
        :param str size: File size.
        :param str sample_ids: Comma separated list sample IDs or UUIDs up to
            a maximum of 100.
        :param str job_id: Job ID that created the file(s) or folder(s).
        :param str annotation: Annotation filters. Example:
            age>30;gender=FEMALE. For more information, please visit
            http://docs.opencb.org/display/opencga/AnnotationSets+1.4.0.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param bool deleted: Boolean to retrieve deleted entries.
        :param str release: Release when it was created.
        :param str field: Field to apply aggregation statistics to (or a list
            of fields separated by semicolons), e.g.:
            studies;type;numSamples[0..10]:1;format:sum(size).
        """

        return self._get(category='files', resource='aggregationStats', **options)

    def load_annotation_sets(self, variable_set_id, path, data=None, **options):
        """
        Load annotation sets from a TSV file.
        PATH: /{apiVersion}/files/annotationSets/load

        :param str path: Path where the TSV file is located in OpenCGA or
            where it should be located. (REQUIRED)
        :param str variable_set_id: Variable set ID or name. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool parents: Flag indicating whether to create parent
            directories if they don't exist (only when TSV file was not
            previously associated).
        :param str annotation_set_id: Annotation set id. If not provided,
            variableSetId will be used.
        :param dict data: JSON containing the 'content' of the TSV file if
            this has not yet been registered into OpenCGA.
        """

        options['variableSetId'] = variable_set_id
        options['path'] = path
        return self._post(category='files', resource='load', subcategory='annotationSets', data=data, **options)

    def bioformats(self, **options):
        """
        List of accepted file bioformats.
        PATH: /{apiVersion}/files/bioformats
        """

        return self._get(category='files', resource='bioformats', **options)

    def create(self, data=None, **options):
        """
        Create file or folder.
        PATH: /{apiVersion}/files/create

        :param dict data: File parameters. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool parents: Create the parent directories if they do not
            exist.
        """

        return self._post(category='files', resource='create', data=data, **options)

    def distinct(self, field, **options):
        """
        File distinct method.
        PATH: /{apiVersion}/files/distinct

        :param str field: Comma separated list of fields for which to obtain
            the distinct values. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str id: Comma separated list of file IDs up to a maximum of
            100. Also admits basic regular expressions using the operator '~',
            i.e. '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i'
            for case insensitive search.
        :param str uuid: Comma separated list file UUIDs up to a maximum of
            100.
        :param str name: Comma separated list of file names. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str path: Comma separated list of paths. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str uri: Comma separated list of uris. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str type: File type, either FILE or DIRECTORY.
        :param str bioformat: Comma separated Bioformat values. For existing
            Bioformats see files/bioformats.
        :param str format: Comma separated Format values. For existing Formats
            see files/formats.
        :param bool external: Boolean field indicating whether to filter by
            external or non external files.
        :param bool resource: Boolean field indicating whether the file is a
            resource or not.
        :param str status: Filter by status.
        :param str internal_status: Filter by internal status.
        :param str internal_variant_index_status: Filter by internal variant
            index status.
        :param str software_name: Software name.
        :param str directory: Directory under which we want to look for files
            or folders.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param str description: Description.
        :param str tags: Tags. Also admits basic regular expressions using the
            operator '~', i.e. '~{perl-regex}' e.g. '~value' for case
            sensitive, '~/value/i' for case insensitive search.
        :param str size: File size.
        :param str sample_ids: Comma separated list sample IDs or UUIDs up to
            a maximum of 100.
        :param str job_id: Job ID that created the file(s) or folder(s).
        :param str annotation: Annotation filters. Example:
            age>30;gender=FEMALE. For more information, please visit
            http://docs.opencb.org/display/opencga/AnnotationSets+1.4.0.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param bool deleted: Boolean to retrieve deleted entries.
        :param str release: Release when it was created.
        """

        options['field'] = field
        return self._get(category='files', resource='distinct', **options)

    def fetch(self, data=None, **options):
        """
        Download an external file to catalog and register it.
        PATH: /{apiVersion}/files/fetch

        :param dict data: Fetch parameters. (REQUIRED)
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_description: Job description.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._post(category='files', resource='fetch', data=data, **options)

    def formats(self, **options):
        """
        List of accepted file formats.
        PATH: /{apiVersion}/files/formats
        """

        return self._get(category='files', resource='formats', **options)

    def link(self, data=None, **options):
        """
        Link an external file into catalog.
        PATH: /{apiVersion}/files/link

        :param dict data: File parameters. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool parents: Create the parent directories if they do not
            exist.
        """

        return self._post(category='files', resource='link', data=data, **options)

    def run_link(self, data=None, **options):
        """
        Link an external file into catalog asynchronously.
        PATH: /{apiVersion}/files/link/run

        :param dict data: File parameters. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='files', resource='run', subcategory='link', data=data, **options)

    def run_postlink(self, data=None, **options):
        """
        Associate non-registered samples for files with high volumes of
            samples.
        PATH: /{apiVersion}/files/postlink/run

        :param dict data: File parameters. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str job_id: Job ID. It must be a unique string within the
            study. An ID will be autogenerated automatically if not provided.
        :param str job_depends_on: Comma separated list of existing job IDs
            the job will depend on.
        :param str job_description: Job description.
        :param str job_tags: Job tags.
        :param str job_scheduled_start_time: Time when the job is scheduled to
            start.
        :param str job_priority: Priority of the job.
        :param bool job_dry_run: Flag indicating that the job will be executed
            in dry-run mode. In this mode, OpenCGA will validate that all
            parameters and prerequisites are correctly set for successful
            execution, but the job will not actually run.
        """

        return self._post(category='files', resource='run', subcategory='postlink', data=data, **options)

    def search(self, **options):
        """
        File search method.
        PATH: /{apiVersion}/files/search

        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param bool count: Get the total number of results matching the query.
            Deactivated by default.
        :param bool flatten_annotations: Boolean indicating to flatten the
            annotations.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str id: Comma separated list of file IDs up to a maximum of
            100. Also admits basic regular expressions using the operator '~',
            i.e. '~{perl-regex}' e.g. '~value' for case sensitive, '~/value/i'
            for case insensitive search.
        :param str uuid: Comma separated list file UUIDs up to a maximum of
            100.
        :param str name: Comma separated list of file names. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str path: Comma separated list of paths. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str uri: Comma separated list of uris. Also admits basic
            regular expressions using the operator '~', i.e. '~{perl-regex}'
            e.g. '~value' for case sensitive, '~/value/i' for case insensitive
            search.
        :param str type: File type, either FILE or DIRECTORY.
        :param str bioformat: Comma separated Bioformat values. For existing
            Bioformats see files/bioformats.
        :param str format: Comma separated Format values. For existing Formats
            see files/formats.
        :param bool external: Boolean field indicating whether to filter by
            external or non external files.
        :param bool resource: Boolean field indicating whether the file is a
            resource or not.
        :param str status: Filter by status.
        :param str internal_status: Filter by internal status.
        :param str internal_variant_index_status: Filter by internal variant
            index status.
        :param str software_name: Software name.
        :param str directory: Directory under which we want to look for files
            or folders.
        :param str creation_date: Creation date. Format: yyyyMMddHHmmss.
            Examples: >2018, 2017-2018, <201805.
        :param str modification_date: Modification date. Format:
            yyyyMMddHHmmss. Examples: >2018, 2017-2018, <201805.
        :param str description: Description.
        :param str tags: Tags. Also admits basic regular expressions using the
            operator '~', i.e. '~{perl-regex}' e.g. '~value' for case
            sensitive, '~/value/i' for case insensitive search.
        :param str size: File size.
        :param str sample_ids: Comma separated list sample IDs or UUIDs up to
            a maximum of 100.
        :param str job_id: Job ID that created the file(s) or folder(s).
        :param str annotation: Annotation filters. Example:
            age>30;gender=FEMALE. For more information, please visit
            http://docs.opencb.org/display/opencga/AnnotationSets+1.4.0.
        :param str acl: Filter entries for which a user has the provided
            permissions. Format: acl={user}:{permissions}. Example:
            acl=john:WRITE,WRITE_ANNOTATIONS will return all entries for which
            user john has both WRITE and WRITE_ANNOTATIONS permissions. Only
            study owners or administrators can query by this field. .
        :param bool deleted: Boolean to retrieve deleted entries.
        :param str release: Release when it was created.
        """

        return self._get(category='files', resource='search', **options)

    def upload(self, **options):
        """
        Resource to upload a file by chunks.
        PATH: /{apiVersion}/files/upload

        :param inputstream file: File to upload.
        :param str name: File name to overwrite the input fileName.
        :param str file_name: [DEPRECATED] File name to overwrite the input
            fileName.
        :param str format: File format. Allowed values: ['VCF BCF GVCF TBI
            BIGWIG SAM BAM BAI CRAM CRAI FASTQ FASTA PED TAB_SEPARATED_VALUES
            COMMA_SEPARATED_VALUES XML PROTOCOL_BUFFER JSON AVRO PARQUET PDF
            IMAGE PLAIN BINARY NONE UNKNOWN']
        :param str file_format: [DEPRECATED] File format. Allowed values:
            ['VCF BCF GVCF TBI BIGWIG SAM BAM BAI CRAM CRAI FASTQ FASTA PED
            TAB_SEPARATED_VALUES COMMA_SEPARATED_VALUES XML PROTOCOL_BUFFER
            JSON AVRO PARQUET PDF IMAGE PLAIN BINARY NONE UNKNOWN']
        :param str bioformat: File bioformat. Allowed values:
            ['MICROARRAY_EXPRESSION_ONECHANNEL_AGILENT
            MICROARRAY_EXPRESSION_ONECHANNEL_AFFYMETRIX
            MICROARRAY_EXPRESSION_ONECHANNEL_GENEPIX
            MICROARRAY_EXPRESSION_TWOCHANNELS_AGILENT
            MICROARRAY_EXPRESSION_TWOCHANNELS_GENEPIX DATAMATRIX_EXPRESSION
            IDLIST IDLIST_RANKED ANNOTATION_GENEVSANNOTATION OTHER_NEWICK
            OTHER_BLAST OTHER_INTERACTION OTHER_GENOTYPE OTHER_PLINK OTHER_VCF
            OTHER_PED VCF4 VARIANT ALIGNMENT COVERAGE SEQUENCE PEDIGREE
            REFERENCE_GENOME NONE UNKNOWN']
        :param str checksum: Expected MD5 file checksum.
        :param bool resource: Boolean field indicating whether the file is a
            resource or not.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str relative_file_path: Path within catalog (directory) where
            the file will be located (default: root folder).
        :param str description: description.
        :param bool parents: Create the parent directories if they do not
            exist.
        """

        return self._post(category='files', resource='upload', **options)

    def acl(self, files, **options):
        """
        Return the acl defined for the file or folder. If member is provided,
            it will only return the acl for the member.
        PATH: /{apiVersion}/files/{files}/acl

        :param str files: Comma separated list of file IDs or names up to a
            maximum of 100. (REQUIRED)
        :param str study: Comma separated list of Studies
            [[organization@]project:]study where study and project can be
            either the ID or UUID up to a maximum of 100.
        :param str member: User or group id.
        :param bool silent: Boolean to retrieve all possible entries that are
            queried for, false to raise an exception whenever one of the
            entries looked for cannot be shown for whichever reason.
        """

        return self._get(category='files', resource='acl', query_id=files, **options)

    def delete(self, files, **options):
        """
        Delete existing files and folders.
        PATH: /{apiVersion}/files/{files}/delete

        :param str files: Comma separated list of file ids, names or paths.
            (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool skip_trash: Skip trash and delete the files/folders from
            disk directly (CANNOT BE RECOVERED).
        """

        return self._delete(category='files', resource='delete', query_id=files, **options)

    def info(self, files, **options):
        """
        File info.
        PATH: /{apiVersion}/files/{files}/info

        :param str files: Comma separated list of file IDs or names up to a
            maximum of 100. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param bool flatten_annotations: Flatten the annotations?.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param bool deleted: Boolean to retrieve deleted files.
        """

        return self._get(category='files', resource='info', query_id=files, **options)

    def unlink(self, files, **options):
        """
        Unlink linked files and folders.
        PATH: /{apiVersion}/files/{files}/unlink

        :param str files: Comma separated list of file ids, names or paths.
            (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._delete(category='files', resource='unlink', query_id=files, **options)

    def update(self, files, data=None, **options):
        """
        Update some file attributes.
        PATH: /{apiVersion}/files/{files}/update

        :param dict data: Parameters to modify. (REQUIRED)
        :param str files: Comma separated list of file ids, names or paths.
            Paths must be separated by : instead of /. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str sample_ids_action: Action to be performed if the array of
            samples is being updated. Allowed values: ['ADD SET REMOVE']
        :param str annotation_sets_action: Action to be performed if the array
            of annotationSets is being updated. Allowed values: ['ADD SET
            REMOVE']
        :param str related_files_action: Action to be performed if the array
            of relatedFiles is being updated. Allowed values: ['ADD SET
            REMOVE']
        :param str tags_action: Action to be performed if the array of tags is
            being updated. Allowed values: ['ADD SET REMOVE']
        """

        return self._post(category='files', resource='update', query_id=files, data=data, **options)

    def update_annotation_sets_annotations(self, file, annotation_set, data=None, **options):
        """
        Update annotations from an annotationSet.
        PATH: /{apiVersion}/files/{file}/annotationSets/{annotationSet}/annotations/update

        :param str annotation_set: AnnotationSet ID to be updated. (REQUIRED)
        :param str file: File id, name or path. Paths must be separated by :
            instead of /. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str action: Action to be performed: ADD to add new annotations;
            REPLACE to replace the value of an already existing annotation; SET
            to set the new list of annotations removing any possible old
            annotations; REMOVE to remove some annotations; RESET to set some
            annotations to the default value configured in the corresponding
            variables of the VariableSet if any. Allowed values: ['ADD SET
            REMOVE RESET REPLACE']
        :param dict data: Json containing the map of annotations when the
            action is ADD, SET or REPLACE, a json with only the key 'remove'
            containing the comma separated variables to be removed as a value
            when the action is REMOVE or a json with only the key 'reset'
            containing the comma separated variables that will be set to the
            default value when the action is RESET.
        """

        return self._post(category='files', resource='annotations/update', query_id=file, subcategory='annotationSets', second_query_id=annotation_set, data=data, **options)

    def download(self, file, **options):
        """
        Download file.
        PATH: /{apiVersion}/files/{file}/download

        :param str file: File id, name or path. Paths must be separated by :
            instead of /. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._get(category='files', resource='download', query_id=file, **options)

    def grep(self, file, **options):
        """
        Filter lines of the file containing the pattern.
        PATH: /{apiVersion}/files/{file}/grep

        :param str file: File uuid, id, or name. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param str pattern: String pattern.
        :param bool ignore_case: Flag to perform a case insensitive search.
        :param int max_count: Stop reading a file after 'n' matching lines. 0
            means no limit.
        """

        return self._get(category='files', resource='grep', query_id=file, **options)

    def head(self, file, **options):
        """
        Show the first lines of a file (up to a limit).
        PATH: /{apiVersion}/files/{file}/head

        :param str file: File uuid, id, or name. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param int offset: Starting byte from which the file will be read.
        :param int lines: Maximum number of lines to be returned up to a
            maximum of 1000.
        """

        return self._get(category='files', resource='head', query_id=file, **options)

    def image(self, file, **options):
        """
        Obtain the base64 content of an image.
        PATH: /{apiVersion}/files/{file}/image

        :param str file: File ID. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._get(category='files', resource='image', query_id=file, **options)

    def move(self, file, data=None, **options):
        """
        Move file to a different path.
        PATH: /{apiVersion}/files/{file}/move

        :param dict data: Parameters to modify. (REQUIRED)
        :param str file: File id, UUID or name. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._post(category='files', resource='move', query_id=file, data=data, **options)

    def refresh(self, file, **options):
        """
        Refresh metadata from the selected file or folder. Return updated
            files.
        PATH: /{apiVersion}/files/{file}/refresh

        :param str file: File id, name or path. Paths must be separated by :
            instead of /. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._get(category='files', resource='refresh', query_id=file, **options)

    def tail(self, file, **options):
        """
        Show the last lines of a file (up to a limit).
        PATH: /{apiVersion}/files/{file}/tail

        :param str file: File uuid, id, or name. (REQUIRED)
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param int lines: Maximum number of lines to be returned up to a
            maximum of 1000.
        """

        return self._get(category='files', resource='tail', query_id=file, **options)

    def list(self, folder, **options):
        """
        List all the files inside the folder.
        PATH: /{apiVersion}/files/{folder}/list

        :param str folder: Folder ID, name or path. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param int limit: Number of results to be returned.
        :param int skip: Number of results to skip.
        :param bool count: Get the total number of results matching the query.
            Deactivated by default.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        """

        return self._get(category='files', resource='list', query_id=folder, **options)

    def tree(self, folder, **options):
        """
        Obtain a tree view of the files and folders within a folder.
        PATH: /{apiVersion}/files/{folder}/tree

        :param str folder: Folder id or name. Paths must be separated by :
            instead of /. (REQUIRED)
        :param str include: Fields included in the response, whole JSON path
            must be provided.
        :param str exclude: Fields excluded in the response, whole JSON path
            must be provided.
        :param str study: Study [[organization@]project:]study where study and
            project can be either the ID or UUID.
        :param int max_depth: Maximum depth to get files from.
        """

        return self._get(category='files', resource='tree', query_id=folder, **options)




import os
import json
import requests
import yaml


class ClientConfiguration(object):
    """PyOpenCGA configuration"""
    
    def __init__(self, config_input):
        """
        :param dict or str config_input: a dict, or a yaml/json file containing an OpenCGA valid client configuration.
        """

        if isinstance(config_input, dict):
            self._config = config_input
        else:
            self._config = self._get_dictionary_from_file(config_input)

        self.get_sso_login_info()

        self._validate_configuration()

    @staticmethod
    def _get_dictionary_from_file(config_fpath):
        config_fhand = open(config_fpath, 'r')

        if config_fpath.endswith('.yml') or config_fpath.endswith('.yaml'):
            config_dict = yaml.safe_load(config_fhand)
        elif config_fpath.endswith('.json'):
            config_dict = json.loads(config_fhand.read())
        else:
            raise NotImplementedError('Input must be a dictionary, a yaml file (.yml or .yaml) or a json file (.json)')

        config_fhand.close()

        return config_dict

    def get_sso_login_info(self):
        if (('sso_login' in self._config and self._config['sso_login']) or
                ('cookies' in self._config and self._config['cookies'])):
            python_session_fhand = open(os.path.expanduser("~/.opencga/session.json"), 'r')
            session_info = json.loads(python_session_fhand.read())
            self._config['sso_login'] = True
            self._config['cookies'] = session_info['cookies']
            self._config['token'] = session_info['token']

    def _validate_configuration(self):
        if self._config is None:
            raise ValueError('Missing configuration dictionary.')

        if 'rest' not in self._config:
            raise ValueError('Missing configuration field "rest".')

        if 'host' not in self._config['rest'] or not self._config['rest']['host']:
            raise ValueError('Missing configuration field "host".')

        self._validate_host(self._config['rest']['host'])

    @staticmethod
    def _validate_host(host):
        try:
            r = requests.head(host, timeout=2)
            if r.status_code == 302:
                return
        except requests.ConnectionError:
            raise Exception('Unreachable host "{}"'.format(host))

    @property
    def configuration(self):
        return self._config

    @property
    def host(self):
        return self._config['rest']['host']

    @host.setter
    def host(self, host):
        self._config['rest']['host'] = host
        self._validate_host(host)

    @property
    def version(self):
        return self._config['version'] if 'version' in self._config else 'v2'

    @version.setter
    def version(self, version):
        self._config['version'] = version

    @property
    def sso_login(self):
        if 'sso_login' in self._config and self._config['sso_login']:
            return self._config['sso_login']
        else:
            return False

    @sso_login.setter
    def sso_login(self, sso_login):
        if isinstance(str, sso_login):
            self._config['sso_login'] = sso_login.lower() == 'true'
        if isinstance(bool, sso_login):
            self._config['sso_login'] = sso_login

    @property
    def cookies(self):
        if 'cookies' in self._config and self._config['cookies']:
            return self._config['cookies']
        else:
            return None

    @cookies.setter
    def cookies(self, cookies):
        self._config['cookies'] = cookies

    @property
    def token(self):
        if 'token' in self._config and self._config['token']:
            return self._config['token']
        else:
            return None

    @token.setter
    def token(self, token):
        self._config['token'] = token

    @property
    def max_attempts(self):
        """Returns configured max_attempts or 1 if not configured"""
        if 'max_attempts' in self._config and self._config['max_attempts']:
            return self._config['max_attempts']
        else:
            return 1

    @max_attempts.setter
    def max_attempts(self, max_attempts):
        self._config['max_attempts'] = max_attempts

    @property
    def min_retry_secs(self):
        """Returns configured min_retry_seconds or 0 if not configured"""
        if 'min_retry_secs' in self._config and self._config['min_retry_secs']:
            return self._config['min_retry_secs']
        else:
            return 0

    @min_retry_secs.setter
    def min_retry_secs(self, min_retry_secs):
        self._config['min_retry_secs'] = min_retry_secs

    @property
    def max_retry_secs(self):
        """Returns configured max_retry_seconds or 0 if not configured"""
        if 'max_retry_secs' in self._config and self._config['max_retry_secs']:
            return self._config['max_retry_secs']
        else:
            return 0

    @max_retry_secs.setter
    def max_retry_secs(self, max_retry_secs):
        self._config['max_retry_secs'] = max_retry_secs



import sys
import pandas as pd

from pyopencga_enterprise.commons import deprecated


class RestResponse:
    def __init__(self, response):
        self.apiVersion = response.get('apiVersion')
        self.time = response.get('time')
        self.events = response.get('events')
        self.params = response.get('params')
        self.responses = response.get('responses')

        # TODO: Remove deprecated response in future release. Added for backwards compatibility
        self.response = response.get('responses')

        # TODO: Remove deprecated result. Added for backwards compatibility
        for query_result in self.responses:
            if 'results' in query_result:
                query_result['result'] = query_result['results']

    @deprecated
    def first(self):
        return self.responses[0]

    def get_results(self, response_pos=0):
        """
        Return the list of results of the response_pos response.
        """
        return self.responses[response_pos]['results']

    def get_result(self, result_pos, response_pos=0):
        """
        Return the result 'result_pos' of the response 'response_pos'.
        """
        return self.responses[response_pos]['results'][result_pos]

    def get_responses(self):
        """
        Return the list of responses
        """
        return self.responses

    def get_response(self, response_pos=0):
        """
        Return the response_pos response.
        """
        return self.responses[response_pos]

    def result_iterator(self, response_pos=None):
        """
        Return all results from all responses as an iterator
        """
        if response_pos is None:
            for response in self.responses:
                for result in response['results']:
                    yield result
        else:
            for result in self.responses[response_pos]['results']:
                yield result

    @staticmethod
    def _get_param_value(result, field):
        items = field.split('.')
        result2 = result
        is_list = False
        for item in items:
            if item in result2:
                if isinstance(result2[item], list):
                    if is_list:
                        return '.'
                    is_list = True
                result2 = result2[item]
            else:
                if is_list:
                    if len(result2) == 0 or isinstance(result2[0][item], list):
                        return '.'
                    result2 = [r[item] for r in result2 if item in r]
                else:
                    return '.'
        if is_list:
            if len(result2) == 0 or isinstance(result2[0], (list, dict)):
                return '{} items'.format(len(result2))
            return ','.join(map(str, set(result2)))
        else:
            return result2

    def print_results(self, fields=None, response_pos=None, limit=None, separator='\t', title=None,
                      metadata=True, outfile=None):
        outfhand = sys.stdout if outfile is None else open(outfile, 'w')
        if title is not None:
            outfhand.write(title + '\n' + '-'*(len(title)+5) + '\n')
        if metadata:
            for event_type in ['INFO', 'WARNING', 'ERROR']:
                for event in self.get_response_events(event_type):
                    msg = '#{}: {}'.format(event['type'], event['description'])
                    if 'id' in event:
                        msg += ' ({})'.format(event['id'])
                    outfhand.write(msg + '\n')
            outfhand.write('#Time: {}\n'.format(self.time))
            outfhand.write('#Num matches: {}\n'.format(self.get_num_matches(response_pos)))
            outfhand.write('#Num results: {}\n'.format(self.get_num_results(response_pos)))
            outfhand.write('#Num inserted: {}\n'.format(self.get_num_inserted(response_pos)))
            outfhand.write('#Num updated: {}\n'.format(self.get_num_updated(response_pos)))
            outfhand.write('#Num deleted: {}\n'.format(self.get_num_deleted(response_pos)))

        responses = [self.get_response(response_pos)] if response_pos is not None else self.get_responses()
        for response in responses:
            if fields:
                fields = fields.split(',')
            elif 'include' in self.params:
                fields = self.params['include'].split(',')
            else:
                if response['results']:
                    fields = response['results'][0].keys()

            limit = limit if limit is not None else len(response['results'])
            if limit:
                outfhand.write('#{}\n'.format(separator.join(fields)))
            for result in response['results'][:limit]:
                values = [self._get_param_value(result, field) for field in fields]
                outfhand.write(separator.join(map(str, values)) + '\n')

    def to_data_frame(self, response_pos=None):
        responses = [self.get_response(response_pos)] if response_pos is not None else self.get_responses()
        return pd.concat([pd.json_normalize(result) for response in responses
                          for result in response['results']]).reset_index(drop=True)

    def get_response_events(self, event_type=None):
        """
        Return response events by name
        """
        event_names = ['INFO', 'WARNING', 'ERROR']
        if event_type is None or self.events is None:
            return self.events or []
        elif event_type in event_names:
            return [event for event in self.events if event['type'] == event_type]
        else:
            msg = 'Argument "type" must be one of the following values: "{}"'
            raise ValueError(msg.format(', '.join(event_names)))

    def get_result_events(self, event_type=None, response_pos=0):
        """Return result events by name and position"""
        event_names = ['INFO', 'WARNING', 'ERROR']
        response = self.responses[response_pos]
        if event_type is None:
            return response['events'] \
                if 'events' in response and response['events'] else []
        elif event_type in event_names:
            return [event for event in response['events'] if event['type'] == event_type] \
                if 'events' in response and response['events'] else []
        else:
            msg = 'Argument "type" must be one of the following values: "{}"'
            raise ValueError(msg.format(', '.join(event_names)))

    def get_num_matches(self, response_pos=None):
        """
        Return number of matches
        """
        if response_pos is not None:
            return self.responses[response_pos]['numMatches']
        else:
            num_matches = 0
            for query_result in self.responses:
                if 'numMatches' in query_result:
                    num_matches += query_result['numMatches']
            return num_matches

    def get_num_results(self, response_pos=None):
        """
        Return number of results
        """
        if response_pos is not None:
            return self.responses[response_pos]['numResults']
        else:
            num_results = 0
            for query_result in self.responses:
                if 'numResults' in query_result:
                    num_results += query_result['numResults']
            return num_results

    def get_num_inserted(self, response_pos=None):
        """
        Return number of inserted
        """
        if response_pos is not None:
            return self.responses[response_pos]['numInserted']
        else:
            num_inserted = 0
            for query_result in self.responses:
                if 'numInserted' in query_result:
                    num_inserted += query_result['numInserted']
            return num_inserted

    def get_num_updated(self, response_pos=None):
        """
        Return number of updated
        """
        if response_pos is not None:
            return self.responses[response_pos]['numUpdated']
        else:
            num_updated = 0
            for query_result in self.responses:
                if 'numUpdated' in query_result:
                    num_updated += query_result['numUpdated']
            return num_updated

    def get_num_deleted(self, response_pos=None):
        """
        Return number of deleted
        """
        if response_pos is not None:
            return self.responses[response_pos]['numDeleted']
        else:
            num_deleted = 0
            for query_result in self.responses:
                if 'numDeleted' in query_result:
                    num_deleted += query_result['numDeleted']
            return num_deleted



import sys
import threading
from time import sleep
import warnings

import requests

from pyopencga_enterprise.exceptions import OpencgaInvalidToken, OpencgaAuthorisationError

try:
    from Queue import Queue
except ImportError:
    from queue import Queue

_CALL_BATCH_SIZE = 2000
_NUM_THREADS_DEFAULT = 4


def deprecated(func):
    """Prints a warning for functions marked as deprecated"""
    def new_func(*args, **kwargs):
        warnings.simplefilter('always', DeprecationWarning)  # turn off filter
        warnings.warn('Call to deprecated function "{}".'.format(func.__name__),
                      category=DeprecationWarning, stacklevel=2)
        warnings.simplefilter('default', DeprecationWarning)  # reset filter
        return func(*args, **kwargs)
    return new_func


def snake_to_camel_case(text):
    """Converts snake_case to camelCase"""
    components = text.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])


def _create_rest_url(host, version, sid, category, resource, subcategory=None, query_id=None,
                     second_query_id=None, options=None):
    """Creates the URL for querying the REST service"""

    # Creating the basic URL
    url = ('/'.join([host,
                     'webservices/rest',
                     version,
                     category
                     ]))

    # If subcategory is queried, query_id can be absent
    if query_id is not None:
        url += '/' + query_id

    if subcategory is not None:
        url += '/' + subcategory

    if second_query_id is not None:
        url += '/' + second_query_id

    url += '/' + resource

    header = {"Accept-Encoding": "gzip"}
    if sid is not None:
        header['Authorization'] = 'Bearer {}'.format(sid)

    # Checking optional params
    if options is not None:
        opts = []
        for k, v in options.items():
            k = snake_to_camel_case(k)
            if k == 'debug':
                continue
            if isinstance(v, list):
                opts.append(k + '=' + ','.join(map(str, v)))
            else:
                opts.append(k + '=' + str(v))
        if opts:
            url += '?' + '&'.join(opts)
    return url, header


def _fetch(config, sid, category, resource, method, subcategory=None, query_id=None,
           second_query_id=None, data=None, options=None):
    """Queries the REST service retrieving results until exhaustion or limit"""
    # HERE BE DRAGONS
    final_response = None

    # Setting up skip and limit default parameters
    call_skip = 0
    call_limit = 1000
    max_limit = None
    if options is None:
        opts = {'skip': call_skip, 'limit': call_limit}
    else:
        opts = options.copy()  # Do not modify original data!
        if 'skip' not in opts:
            opts['skip'] = call_skip
        # If 'limit' is specified, a maximum of 'limit' results will be returned
        if 'limit' in opts:
            max_limit = int(opts['limit'])
        # Server must be always queried for results in groups of 1000
        opts['limit'] = call_limit

    # If there is a query_id, the next variables will be used
    total_id_list = []  # All initial ids
    next_id_list = []  # Ids which should be queried again for more results
    next_id_indexes = []  # Ids position in the final response
    if query_id is not None:
        total_id_list = query_id.split(',')

    # If some query has more than 'call_limit' results, the server will be
    # queried again to retrieve the next 'call_limit results'
    call = True
    current_query_id = None  # Current REST query
    current_id_list = None  # Current list of ids
    time_out_counter = 0  # Number of times a query is repeated due to time-out
    while call:
        # Check 'limit' parameter if there is a maximum limit of results
        if max_limit is not None and max_limit <= call_limit:
            opts['limit'] = max_limit

        # Updating query_id and list of ids to query
        if query_id is not None:
            if current_query_id is None:
                current_query_id = query_id
                current_id_list = total_id_list
                current_id_indexes = range(len(total_id_list))
            else:
                current_query_id = ','.join(next_id_list)
                current_id_list = next_id_list
                current_id_indexes = next_id_indexes

        # Retrieving url
        url, header = _create_rest_url(host=config.host,
                                       version=config.version,
                                       category=category,
                                       sid=sid,
                                       subcategory=subcategory,
                                       query_id=current_query_id,
                                       second_query_id=second_query_id,
                                       resource=resource,
                                       options=opts)

        # DEBUG param
        if opts is not None and 'debug' in opts and opts['debug']:
            sys.stderr.write(url + '\n')

        # Getting REST response
        if method == 'get':
            try:
                r = requests.get(url, headers=header, cookies=config.cookies)
            except requests.exceptions.ConnectionError:
                sleep(1)
                r = requests.get(url, headers=header, cookies=config.cookies)
        elif method == 'post':
            try:
                r = requests.post(url, json=data, headers=header, cookies=config.cookies)
            except requests.exceptions.ConnectionError:
                sleep(1)
                r = requests.post(url, json=data, headers=header, cookies=config.cookies)
        elif method == 'delete':
            try:
                r = requests.delete(url, headers=header, cookies=config.cookies)
            except requests.exceptions.ConnectionError:
                sleep(1)
                r = requests.delete(url, headers=header, cookies=config.cookies)
        else:
            raise NotImplementedError('method: ' + method + ' not implemented.')

        if r.status_code == 504:  # Gateway Time-out
            if time_out_counter == 99:
                msg = 'Server not responding in time'
                raise requests.ConnectionError(msg)
            time_out_counter += 1
            continue
        time_out_counter = 0

        if r.status_code == 401:
            raise OpencgaInvalidToken(r.content)
        elif r.status_code == 403:
            raise OpencgaAuthorisationError(r.content)

        elif r.status_code != 200:
            raise Exception(r.content)

        try:
            response = r.json()

            # TODO Remove deprecated response and result in future release. Added for backwards compatibility
            if 'response' in response:
                response['responses'] = response['response']
            for query_result in response['responses']:
                if 'result' in query_result:
                    query_result['results'] = query_result['result']

        except ValueError:
            msg = 'Bad JSON format retrieved from server'
            raise ValueError(msg)

        # Setting up final_response
        if final_response is None:
            final_response = response
        # Concatenating results
        else:
            if query_id is not None:
                for index, res in enumerate(response['responses']):
                    id_index = current_id_indexes[index]
                    final_response[id_index]['results'] += res['results']
            else:
                final_response['responses'][0]['results'] += response['responses'][0]['results']

        if query_id is not None:
            # Checking which ids are completely retrieved
            next_id_list = []
            next_id_indexes = []
            for index, res in enumerate(response['responses']):
                if res['numResults'] == call_limit:
                    next_id_list.append(current_id_list[index])
                    next_id_indexes.append(current_id_indexes[index])
            # Ending REST calling when there are no more ids to retrieve
            if not next_id_list:
                call = False
        else:
            # Ending REST calling when there are no more results to retrieve
            if response['responses'][0]['numResults'] != call_limit:
                call = False

        # Skipping the first 'limit' results to retrieve the next ones
        opts['skip'] += call_limit

        # Subtracting the number of returned results from the maximum goal
        if max_limit is not None:
            max_limit -= call_limit
            # When 'limit' is 0 returns all the results. So, break the loop if 0
            if max_limit == 0:
                break

    return final_response


def _worker(queue, results, config, sid, category, resource, method, subcategory=None,
            second_query_id=None, data=None, options=None):

    """Manages the queue system for the threads"""
    while True:
        # Fetching new element from the queue
        index, query_id = queue.get()
        response = _fetch(config=config, sid=sid, category=category, subcategory=subcategory,
                          resource=resource, method=method, data=data, query_id=query_id,
                          second_query_id=second_query_id, options=options)
        # Store data in results at correct index
        results[index] = response
        # Signaling to the queue that task has been processed
        queue.task_done()


def merge_query_responses(query_response_list):
    final_response = query_response_list[0]
    for i, query_response in enumerate(query_response_list):
        if i != 0:
            final_response['events'] += query_response['events']
            final_response['time'] += query_response['time']
            # final_response['responses'] += response['responses']

            for key in query_response['params']:
                if final_response['params'][key] != query_response['params'][key]:
                    final_response['params'][key] += ',' + query_response['params'][key]

            for j, query_result in enumerate(query_response['responses']):
                if len(final_response['responses'])-1 < j:
                    final_response['responses'] += []
                for key in query_result:
                    if key not in final_response['responses'][j]:
                        final_response['responses'][j][key] = query_result[key]
                    else:
                        if isinstance(query_result[key], (int, list)):
                            final_response['responses'][j][key] += query_result[key]
    return final_response


def execute(config, sid, category, resource, method, subcategory=None, query_id=None,
            second_query_id=None, data=None, options=None):
    """Queries the REST service using multiple threads if needed"""

    # If query_id is an array, convert to comma-separated string
    if query_id is not None:
        if isinstance(query_id, list):
            query_id = ','.join([str(item) for item in query_id])
        else:
            query_id = str(query_id)  # convert to string so we can call this method with int ids

    # Multithread if the number of queries is greater than _CALL_BATCH_SIZE
    if query_id is None or len(query_id.split(',')) <= _CALL_BATCH_SIZE:
        response = _fetch(config=config, sid=sid, category=category, subcategory=subcategory,
                          resource=resource, method=method, data=data, query_id=query_id,
                          second_query_id=second_query_id, options=options)
        return response
    else:
        if options is not None and 'num_threads' in options:
            num_threads = options['num_threads']
        else:
            num_threads = _NUM_THREADS_DEFAULT

        # Splitting query_id into batches depending on the call batch size
        id_list = query_id.split(',')
        id_batches = [','.join(id_list[x:x + _CALL_BATCH_SIZE])
                      for x in range(0, len(id_list), _CALL_BATCH_SIZE)]

        # Setting up the queue to hold all the id batches
        q = Queue(maxsize=0)
        # Creating a size defined list to store thread results
        res = [''] * len(id_batches)

        # Setting up the threads
        for thread in range(num_threads):
            t = threading.Thread(target=_worker,
                                 kwargs={'queue': q,
                                         'results': res,
                                         'config': config,
                                         'sid': sid,
                                         'category': category,
                                         'subcategory': subcategory,
                                         'second_query_id': second_query_id,
                                         'resource': resource,
                                         'method': method,
                                         'data': data,
                                         'options': options})
            # Setting threads as "daemon" allows main program to exit eventually
            # even if these do not finish correctly
            t.setDaemon(True)
            t.start()

        # Loading up the queue with index and id batches for each job
        for index, batch in enumerate(id_batches):
            q.put((index, batch))  # Notice this is a tuple

        # Waiting until the queue has been processed
        q.join()

    # Joining all the responses into a one final response
    final_query_response = merge_query_responses(res)

    return final_query_response





try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pyopencga_enterprise',
    version='3.1.0.dev22',
    description='A REST client for OpenCGA enterprise REST web services',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    packages=['pyopencga_enterprise', 'pyopencga_enterprise.rest_clients'],
    license='Apache Software License',
    author='Daniel Perez-Gil',
    author_email='daniel.perez@zettagenomics.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='zetta-genomics opencga-enterprise bioinformatics genomic database',
    install_requires=[
        'requests >= 2.7',
        'pip >= 7.1.2',
        'pathlib >= 1.0.1',
        'pyyaml >= 3.12',
        'pandas >= 1.1.5',
        'flask >= 2.0.3'
    ],
    project_urls={
        'OpenCGA-enterprise': 'https://github.com/zetta-genomics/opencga-enterprise',
        'Bug Reports': 'https://github.com/zetta-genomics/opencga-enterprise/issues',
    }
)
